"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CodeSigningPlugin = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _webpack = _interopRequireDefault(require("webpack"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class CodeSigningPlugin {
  /**
   * Constructs new `RepackPlugin`.
   *
   * @param config Plugin configuration options.
   */
  constructor(config) {
    this.config = config;
    this.config.outputFile = this.config.outputFile ?? 'code_signing_mapping.json';
    this.config.privateKeyPath = this.config.privateKeyPath ?? './private.pem';
  }
  /**
   * Apply the plugin.
   *
   * @param compiler Webpack compiler instance.
   */


  apply(compiler) {
    const pluginName = CodeSigningPlugin.name;

    const privateKeyPath = _path.default.join(compiler.context, this.config.privateKeyPath);

    const privateKey = _fsExtra.default.readFileSync(privateKeyPath); // Tapping to the "thisCompilation" hook in order to further tap
    // to the compilation process on an later stage.


    compiler.hooks.thisCompilation.tap(pluginName, compilation => {
      // we need to make sure that assets are fully processed in order
      // to create a code-signing mapping.
      compilation.hooks.afterProcessAssets.tap(pluginName, assets => {
        // "assets" is an object that contains all assets
        // in the compilation, the keys of the object are pathnames of the assets
        // and the values are file sources.
        const chunkFiles = new Set(); // adjust for chunk name to filename

        compilation.chunks.forEach(chunk => {
          chunk.files.forEach(file => {
            var _this$config$excludeC;

            // Exclude main output bundle because it's always local
            if (file === compilation.outputOptions.filename) {
              return;
            } // Exclude chunks specified in config


            if ((_this$config$excludeC = this.config.excludeChunks) !== null && _this$config$excludeC !== void 0 && _this$config$excludeC.includes(String(chunk.id))) {
              return;
            }

            chunkFiles.add(file);
          });
        });
        const content = Object.entries(assets).filter(([fileName]) => chunkFiles.has(fileName)).reduce((acc, [fileName, file]) => {
          // get bundle
          const bundle = file.source(); // generate bundle hash

          const hash = _crypto.default.createHash('sha256').update(bundle).digest('hex'); // generate token


          const token = _jsonwebtoken.default.sign({
            hash
          }, privateKey, {
            algorithm: 'RS256'
          });

          acc[fileName] = token;
          return acc;
        }, {});
        const json = JSON.stringify(content);

        if (this.config.outputPath) {
          _fsExtra.default.ensureDirSync(this.config.outputPath);

          _fsExtra.default.writeFileSync(_path.default.join(compiler.context, this.config.outputPath, this.config.outputFile), json);
        } else {
          compilation.emitAsset(this.config.outputFile, new _webpack.default.sources.RawSource(json));
        }
      });
    });
  }

}

exports.CodeSigningPlugin = CodeSigningPlugin;
//# sourceMappingURL=CodeSigningPlugin.js.map